function [xx,yy,im] = findworm(img,varargin)
% FINDWORM - Find a worm in an image
%    [xx,yy] = FINDWORM(img,thr)

kv = getopt('thresh=.3 pwr=[] lambda4=[] sigperp=[] siglong=[] istep=2 iscale=4 irange=2 ndir=8 interp=0 minlen=30 plotflag=0 ondark=0 onlight=0 softmax=0 exclude={}',varargin);

if isempty(kv.lambda4) & isempty(kv.sigperp)
  kv.lambda4 = 2;
end

[Y X]=size(img);

%% t_started = toc;

if iscell(img)
  imax=img{1};
  dirs=img{2};
else
  fprintf(1,'Finding local directionality...              \r');
  %img = fw_doexclude(img,kv.exclude);
  [imax,dirs] = directimg2(img,kv);
  %% t_directed = toc;
  imax = fw_doexclude(imax,kv.exclude);
  %% t_excluded = toc;
end

if kv.plotflag
  figure; clf
  subplot(1,2,1);
  imagesc(img); colormap(gray);
  subplot(1,2,2);
  plotmaxdir(imax,-2*dirs+pi,1,.5);
end
%% t_plotted = toc;

% Find the abs. max, and work in both dirs from there, by
% calculating directions and stepping 10 pix in that direction, then
% find local max within 10 pix from there in the normal direction.
[xx,yy,im] = findbydirection(imax,dirs,kv);

%% t_traced = toc;

if kv.plotflag
  subplot(1,2,1);
  hold on
  if kv.ondark
    clr='r';
  else
    clr='g';
  end
  plot(xx,yy,clr);
  plot(xx(1),yy(1),[clr '.'])
  plot(xx(end),yy(end),[clr '.'])
  subplot(1,2,2);
  hold on
  plot(xx,yy,'w:');e
  plot(xx(1),yy(1),'w.');
  plot(xx(end),yy(end),'w.');
end
%% t_plotted2 = toc;

if kv.interp
  fprintf(1,'Interpolating to %i points...              \r',kv.interp);
  if isempty(xx)
    xx=zeros(kv.interp,1)+nan;
    yy=zeros(kv.interp,1)+nan;
    im=zeros(kv.interp,1)+nan;
  else
    [xx,yy,im] = reinterpcurve(xx,yy,kv.interp,im);
  end
end
%% t_interped = toc;
fprintf(1,'Findworm done.                                  \n');
%% fd=fopen('/tmp/timing.txt','a');
%% fprintf(fd,'%.3f %.3f %.3f %.3f %.3f %.3f %.3f\n',...
%%     t_started,t_directed,t_excluded,t_plotted,...
%%     t_traced,t_plotted2,t_interped);
%% fclose(fd);

function img = fw_doexclude(img,rect,val)
K=size(rect,1);
if nargin<3
  val=0;
end
for k=1:K
  img(rect(k,3):rect(k,4),rect(k,1):rect(k,2))=val;
end
